from generate_analogvariables import *
from generate_digitalvariables import *
from generate_commands import *
from generate_switches import *
from generate_userpins import *
from datetime import datetime


def save_header_file(filename, contents, folder, is_template=False):
    tpl = """// Auto-generated [istemplate]file: [filename]
// Generation timestamp: [timestamp]

#ifndef [preprocname]
#define [preprocname]

[contents]

#endif // [preprocname]
"""

    preprocname = '_AUTOGENERATED_' + filename.upper().replace('.', '_')
    file_contents = tpl
    file_contents = file_contents.replace('[filename]', filename)
    file_contents = file_contents.replace('[timestamp]', timestamp_str)
    file_contents = file_contents.replace('[preprocname]', preprocname)
    file_contents = file_contents.replace('[contents]', contents)
    file_contents = file_contents.replace('[istemplate]', '' if not is_template else 'template ')

    with open(folder + filename, "w") as f:
        f.write(file_contents)
    print 'file written to "{}"'.format(folder + filename)


def save_source_file(filename, contents, folder, is_template=False):
    tpl = """// Auto-generated [istemplate]file: [filename]
// Generation timestamp: [timestamp]

[contents]
"""
    file_contents = tpl
    file_contents = file_contents.replace('[filename]', filename)
    file_contents = file_contents.replace('[timestamp]', timestamp_str)
    file_contents = file_contents.replace('[contents]', contents)
    file_contents = file_contents.replace('[istemplate]', '' if not is_template else 'template ')

    with open(folder + filename, "w") as f:
        f.write(file_contents)
    print 'file written to "{}"'.format(folder + filename)


if __name__ == '__main__':

    timestamp_str = str(datetime.now())

    firmware_folder = '../../../micro/BPD-V01-FIRMWARE.X'
    output_folder_sources = firmware_folder + '/generated/sources/'
    output_folder_templates = './output/templates/'
    output_folder_templates_commands = './output/templates/commands/'

    # create list of variables used in code
    switches = SwitchList()

    analogvariables = AnalogVariableList(
        _var_enum_index_start=1,
        _alarm_enum_index_start=1)

    digitalvariables = DigitalVariableList(
        _var_enum_index_start=max(analogvariables.get_var_enum_indexes())+1,
        _alarm_enum_index_start=max(analogvariables.get_alarm_enum_indexes())+1)

    # print analogvariables.get_var_enum_indexes()
    # print digitalvariables.get_var_enum_indexes()
    # print analogvariables.get_alarm_enum_indexes()
    # print digitalvariables.get_alarm_enum_indexes()

    userpins = UserPinList()

    commands = CommandList(switches, analogvariables, digitalvariables, userpins)

    ####################################################################################################################
    # generate the header file variables.h
    filename = 'variables.h'
    contents = ''
    # analog variables
    contents += generate_analogvariables_header(analogvariables)
    contents += generate_analogalarm_callbacks_header(analogvariables)
    # digital variables
    contents += generate_digitalvariables_header(digitalvariables)
    contents += generate_digitalalarm_callbacks_header(digitalvariables)
    # switches
    contents += generate_switches_header(switches)
    # user pins
    contents += generate_userpins_header(userpins)
    # save to file
    save_header_file(filename, contents, output_folder_sources)

    ####################################################################################################################
    # generate the source file variables.c
    filename = 'variables.c'
    contents = ''
    # analog variables
    contents += generate_analogvariables_source(analogvariables)
    # digital variables
    contents += generate_digitalvariables_source(digitalvariables)
    # switches
    contents += generate_switches_source(switches, PayloadFieldEnum.get_persistent_list()['SWITCHNUM'])
    # user pins
    contents += generate_userpins_source(userpins, PayloadFieldEnum.get_persistent_list()['PINNUM'])
    # save to file
    save_source_file(filename, contents, output_folder_sources)

    ####################################################################################################################
    # generate the template source file for the alarm callback list
    filename = 'alarm_callbacks.tpl.c'
    contents = ''
    # analog variables
    contents += generate_analogalarm_callbacks_source_template(analogvariables)
    # digital variables
    contents += generate_digitalalarm_callbacks_source_template(digitalvariables)
    # save to file
    save_source_file(filename, contents, output_folder_templates, is_template=True)

    ####################################################################################################################
    # generate the command header file commandstable.h
    filename = 'commandstable.h'
    contents = ''
    # analog variables
    contents += generate_commandtable_header(commands)
    # save to file
    save_header_file(filename, contents, output_folder_sources)

    ####################################################################################################################
    # generate the type definition header file datatypes.h
    filename = 'datatypes.h'
    contents = ''
    # analog variables
    contents += generate_commands_typedefs()
    # save to file
    save_header_file(filename, contents, output_folder_sources)

    ####################################################################################################################
    # generate the command table source file commandstable.c
    filename = 'commandstable.c'
    contents = ''
    # analog variables
    contents += generate_commandtable_sources(commands)
    # save to file
    save_source_file(filename, contents, output_folder_sources)

    ####################################################################################################################
    # generate the commands callback template files
    for command in commands.entries():
        if command.implement:
            contents = generate_command_callback_template(command)
            filename = 'command_{}.tpl.c'.format(command.name)
            save_source_file(filename, contents, output_folder_templates_commands, is_template=True)









