#!/usr/bin/python
"""
Generates a list of the available BPS commands
"""

import sys
import os
sys.path.insert(0, '../codegen')

import commands
from entities.descriptors import PayloadFieldEnum
from utils import mls


def print_field_description(_field, _field_idx):
    print '          FIELD #{} ({}) : {}, {}'.format(mls(str(_field_idx), 2), mls(_field.ctype, 8), _field.name, _field.description)
    if isinstance(_field, PayloadFieldEnum):
        print '              {} indexing:'.format(_field.name)
        for index in _field.index_list:
            print '                  [{}] --> {}'.format(index, _field.get_value_by_index(index))


if __name__ == '__main__':

    # redirect stdout to null in order to avoid the warnings during object creation
    stdout = sys.stdout
    f = open(os.devnull, 'w')
    sys.stdout = f

    switch_list = commands.SwitchList()

    # analog_variable_list = commands.AnalogVariableList()
    # digital_variable_list = commands.DigitalVariableList()

    analog_variable_list = commands.AnalogVariableList(
        _var_enum_index_start=1,
        _alarm_enum_index_start=1)

    digital_variable_list = commands.DigitalVariableList(
        _var_enum_index_start=max(analog_variable_list.get_var_enum_indexes())+1,
        _alarm_enum_index_start=max(analog_variable_list.get_alarm_enum_indexes())+1)

    user_pin_list = commands.UserPinList()
    commands = commands.CommandList(switch_list, analog_variable_list, digital_variable_list, user_pin_list)

    # restore normal stdout
    sys.stdout = stdout

    if len(sys.argv) == 1:
        print "List of available commands:"
        for command in commands.entries():
            print '  ' + mls(command.name, 32), command.description
        print 'Type help.py COMMAND to obtain a detailed description, or help.py ALL to show details for all commands.'
    elif len(sys.argv) == 2:
        command_name = sys.argv[1].upper()
        if command_name == 'ALL':
            filtered_commands = commands.entries()
        else:
            filtered_commands = [command for command in commands.entries() if command.name.startswith(command_name)]
        print 'Details of commands starting with "{}":'.format(command_name)

        if not filtered_commands:
            print 'List is empty.'
        for command in filtered_commands:
            print ''
            print '  COMMAND NAME : "{}"'.format(command.name)
            print '      DESCRIPTION      : {}'.format(command.description)
            print '      REQUEST CODE     : {}'.format(command.request_code)
            print '      RESPONSE CODE    : {}'.format(command.response_code)
            print '      REQUEST PAYLOAD  :'
            print '          LENGTH : {} fields, {} bytes'.format(len(command.request_payload), command.get_request_payload_len())
            field_idx = 0
            for field in command.request_payload:
                print_field_description(field, field_idx)
                field_idx += 1
            print '      RESPONSE PAYLOAD :'
            print '          LENGTH : {} fields, {} bytes'.format(len(command.response_payload), command.get_response_payload_len())
            field_idx = 0
            for field in command.response_payload:
                print_field_description(field, field_idx)
                field_idx += 1

    print ''
